/*
	EZ-USB(AN2135SC) JTAG / CCLK Probe Interface  Version 1.0

	Copyright(c) 2002 Tatsuyuki Satoh

	2001. 3.18 1st release

	support probe
		Cypress AN2135SC(EZ-USB) without EEPROM
		+ Cypress EZ-USB DevKit(Device Driver)

	I/O port assign for JTAG mode

		FWR      -> TCK
		D[0]     -> TDI
		D[1]     -> TMS
		PC[1]    <- TDO
		PC[3]    <- Target VCC 3.3V

	I/O port assign for SlaveSerial mode

		FWR      -> CCLK
		D[0]     -> DIN
		PC[0]    -> CF_PROGRAM
		PC[1]    <- CF_DONE
		PC[3]    <- Target VCC 3.3V

*/
#include <windows.h>
#include <stdio.h>
#include "isp_if.h"
#include "isp_par.h"

// Warning fix for EZ-USB header file with MINGW
#if !defined(_BYTE_DEFINED)
#define _BYTE_DEFINED
#define _WORD_DEFINED
#endif

// EZ-USN & chamereonUSB General-purpose FirmWare */
#include "cusb.h"
#include "gpfw.h"

int usb_jtag_init(DWORD port_address);
int usb_jtag_power(void);
int usb_jtag_txrx(const BYTE *ibuf,BYTE *obuf,DWORD drsize,int tms_s,int tms_e);
void usb_jtag_end(void);

int usb_ss_init(DWORD port_address);
int usb_ss_power(void);
int usb_ss_tx(const BYTE *data,DWORD size);
int usb_ss_program(BYTE level);
int usb_ss_done(void);
void usb_ss_end(void);
/* -------------------- Driver structure -------------------- */
ISP_PROBE ezusb_probe =
{
//JTAG
	usb_jtag_init,
	usb_jtag_power,
	usb_jtag_txrx,
	usb_jtag_end,
//SS
	usb_ss_init,
	usb_ss_power,
	usb_ss_tx,
	usb_ss_program,
	usb_ss_done,
	usb_ss_end
};
/* ---------------------------------------------------------- */

/* ---------- Hardware pin assign ---------- */

#define CF_PROGRAM    0x01       /* PC0 */
#define CF_TDI        0x01       /* D0  */
#define CF_DIN        0x01       /* D0  */
#define CF_TMS        0x02       /* D1  */
#define CF_TDO(port)  ((port)&2) /* PC1 */
#define CF_DONE(port) ((port)&2) /* PC1 */
#define CF_VCC(port)  ((port)&8) /* PC3 */

// Pxɓ]őoCg
#define TX_MAX 65536*4 //(2048*1024)
#define TXRX_MAX 1

// handle for EZ-USB
static HANDLE usb_h = NULL;

// USB command buffer
static u8 cmd_buf[64];

// USB FirstWrite buffer
static u8 usb_buf[TX_MAX];

/* -------------------- chamereonUSB General-purpose FirmWare interface -------------------- */

// Write to CPIPE (command)
static void usb_cpipe_write(void *buf,DWORD size)
{
	usb_bulk_write(&usb_h,GPFW_CPIPE,buf,size);
}

// Read from RPIPE (read data)
static void usb_rpipe_read(void *buf,DWORD size)
{
	usb_bulk_read(&usb_h,GPFW_RPIPE,buf,size);
}

// Fast WRTITE Transfer
static void usb_fwrite(u8 gpio,void *data,DWORD size)
{
	u8 cmd[4];

	cmd[0]=GPFW_BWRITE | (gpio&0x0f);
	cmd[1]=(u8)(size & 0x3f);
	cmd[2]=(u8)(size >> 6);
	cmd[3]=(u8)(size >> 14);
	usb_bulk_write(&usb_h,GPFW_CPIPE,cmd,4);
	usb_bulk_write(&usb_h,GPFW_WPIPE,data,size);
}

// Initialize
static int ezusb_init(void)
{
	// open EZ-USB & download chanereon usb firmware
	printf("Setup EZ-USB with chamereonUSB GP firnware V1.00\n");
	if(cusb_init(0,&usb_h,fw_bin,"GPFW","V100"))
	{
		usb_h = NULL;
		printf("Can't found EZ-USB\n");
		return 2;
	}
	return 0;
}

/* -------------------- JTAG contyroll -------------------- */

int usb_jtag_init(DWORD port_address)
{
	// initialize EZ-USB
	if( ezusb_init() )
		return 1;

	// setup GPIO & read Target VCC port
	cmd_buf[0]=GPFW_DIR | CF_PROGRAM;
	cmd_buf[1]=GPFW_SET;
	cmd_buf[2]=GPFW_BUS;  // streach=0
	cmd_buf[3]= 0x12;     // RMOD = 10 , TMOD = 10
	usb_cpipe_write(cmd_buf,4); // init & sense

	return 0;
}

int usb_jtag_power(void)
{
	// setup GPIO & read Target VCC port
	cmd_buf[0]=GPFW_GET;
	usb_cpipe_write(cmd_buf,1);
	usb_rpipe_read(cmd_buf,1);

	//Target VCC`FbN
	return CF_VCC(cmd_buf[0]) ? 1 : 0;
}

int usb_jtag_txrx(const BYTE *ibuf,BYTE *obuf,DWORD drsize,int tms_s,int tms_e)
{
	int c;
	int pos;
	BYTE tmsval;
	DWORD block_pos;
	DWORD block_size;

	// USB first write mode ?
	if(obuf == NULL)
	{
		// TMS̒lobt@ɍ쐬
		memset(usb_buf,tms_s ? 0x02 : 0x00,drsize-1);
		usb_buf[drsize-1] = tms_e ? CF_TMS : 0x00;
		// TDI̒lobt@ɍ쐬
		if(ibuf)
		{
			for(pos=0 ; pos<drsize ; pos+=8)
			{
				BYTE value = ibuf[pos/8];
				// nibble -> bitϊ , obt@쐬
				if(value&0x80) usb_buf[pos+0] |= CF_TDI;
				if(value&0x40) usb_buf[pos+1] |= CF_TDI;
				if(value&0x20) usb_buf[pos+2] |= CF_TDI;
				if(value&0x10) usb_buf[pos+3] |= CF_TDI;
				if(value&0x08) usb_buf[pos+4] |= CF_TDI;
				if(value&0x04) usb_buf[pos+5] |= CF_TDI;
				if(value&0x02) usb_buf[pos+6] |= CF_TDI;
				if(value&0x01) usb_buf[pos+7] |= CF_TDI;
			}
		}
		//M
		usb_fwrite(0,usb_buf,drsize);
	}
	else
	{
		//clear outbuf
		if(obuf)
			for(c=0;c<(drsize+7)/8;c++)
				obuf[c] = 0;
		for(block_pos=0;block_pos<drsize;block_pos+=block_size)
		{
			//̑MTCY̐ݒ
			block_size = (block_pos+TXRX_MAX) < drsize ? TXRX_MAX : drsize-block_pos;
			//Mf[^쐬
			for(c=block_pos;c<block_pos+block_size;c++)
			{
				BYTE tdi,tms;

				tdi = ibuf && (ibuf[c/8]&(0x80>>(c&7))) ? 1 : 0;
				tms = (c==drsize-1) ? tms_e : tms_s;
				usb_buf[(c-block_pos)*3+0] = GPFW_WRITE;
				usb_buf[(c-block_pos)*3+1] = (tdi ? 1 : 0) | (tms ? 2 : 0);
				usb_buf[(c-block_pos)*3+2] = GPFW_GET;
			}
			usb_cpipe_write(usb_buf,block_size*3); // TX data
			usb_rpipe_read(usb_buf,block_size);    // read TDO

			// TDOf[^̕ϊ
			for(c=block_pos;c<block_pos+block_size;c++)
			{
				if( CF_TDO(usb_buf[c-block_pos]) )
					obuf[c/8]  |= 0x01<<(c&7);
			}
		}
	}
}

void usb_jtag_end(void)
{
	if(usb_h)
		usb_close(&usb_h);
}

/* -------------------- CCLK contyroll -------------------- */

// initialize PROBE
int usb_ss_init(DWORD port_address)
{
	return usb_jtag_init(port_address);
}

int usb_ss_power(void)
{
	return usb_jtag_power();
}

int usb_ss_tx(const BYTE *data,DWORD size)
{
	int pos;

	// PubN̑Mf[^쐬
	for(pos=0 ; pos<size ; pos++)
	{
		BYTE value = data[pos];
		// nibble -> bitϊ , obt@쐬
		if(value&0x80) usb_buf[pos*8+0] |= CF_DIN;
		if(value&0x40) usb_buf[pos*8+1] |= CF_DIN;
		if(value&0x20) usb_buf[pos*8+2] |= CF_DIN;
		if(value&0x10) usb_buf[pos*8+3] |= CF_DIN;
		if(value&0x08) usb_buf[pos*8+4] |= CF_DIN;
		if(value&0x04) usb_buf[pos*8+5] |= CF_DIN;
		if(value&0x02) usb_buf[pos*8+6] |= CF_DIN;
		if(value&0x01) usb_buf[pos*8+7] |= CF_DIN;
	}
	//PubNAo[Xg]B
	usb_fwrite(CF_PROGRAM,usb_buf,size*8);
	return 0;
}

// set PROGRAM pin level
int usb_ss_program(BYTE level)
{
	// PROGRAMskɂB
	cmd_buf[0] = level ? GPFW_SET | CF_PROGRAM : GPFW_SET;
	usb_cpipe_write(cmd_buf,1);
	return 0;
}

// return DONE pin level
int usb_ss_done(void)
{
	cmd_buf[0]=GPFW_GET | CF_PROGRAM;
	usb_cpipe_write(cmd_buf,1);
	usb_rpipe_read(cmd_buf,1);
	return CF_DONE(cmd_buf[0]) ? 1 : 0;
}

void usb_ss_end(void)
{
	if(usb_h)
		usb_close(&usb_h);
}
